/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Sun Microsystems, Inc.
 *
 * OpenOffice.org - a multi-platform office productivity suite
 *
 * $RCSfile: animationstate.hxx,v $
 * $Revision: 1.5 $
 *
 * This file is part of OpenOffice.org.
 *
 * OpenOffice.org is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * OpenOffice.org is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with OpenOffice.org.  If not, see
 * <http://www.openoffice.org/license.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#ifndef _SDR_MEDIA_MEDIAMANAGER_HXX
#define _SDR_MEDIA_MEDIAMANAGER_HXX

#include <hash_map>
#include <com/sun/star/embed/XStorage.hpp>
#include <comphelper/weak.hxx>
#include <svtools/lstner.hxx>
#include <svx/sdr/media/medialink.hxx>

class SfxObjectShell;
class SvStream;

//////////////////////////////////////////////////////////////////////////////
// predeclarations

namespace sdr {	namespace media	{

typedef std::hash_map< ::rtl::OUString, MediaLinkRef, ::rtl::OUStringHash, ::std::equal_to< ::rtl::OUString > > MediaLinkMap;

class SVX_DLLPUBLIC MediaManager : public ::cppu::OWeakObject, public SfxListener
{
    friend class MediaLink;
public:
    MediaManager( SfxObjectShell* pObjectShell );
    virtual ~MediaManager();
    
    void Dispose();
    
    MediaLinkRef    insertMediaStream( const rtl::OUString& rURL );
    MediaLinkRef    insertMediaStream( const rtl::OUString& rURL, SvStream& rSrcStream, sal_uInt32 nBytes );
    
    MediaLinkRef    getMediaLink( const rtl::OUString& rURL );
    
	virtual void Notify( SfxBroadcaster& rBC, const SfxHint& rHint );

private:
	void SwitchPersistance(const ::com::sun::star::uno::Reference< ::com::sun::star::embed::XStorage >& xStorage );

	void freeMediaLink( const MediaLinkRef& xLink );
    
    rtl::OUString createTemporaryCopy( const rtl::OUString& rURL );
    
	const com::sun::star::uno::Reference < com::sun::star::embed::XStorage >& getDocumentStorage();
	com::sun::star::uno::Reference < com::sun::star::embed::XStorage > getMediaStorage();
private:
	SfxObjectShell* mpObjectShell;
    MediaLinkMap maMediaLinkMap;
    const rtl::OUString msPackagePrefix;
	const rtl::OUString msMedia;
	const rtl::OUString msMediaType;
	const rtl::OUString msStorageMediaType;
	com::sun::star::uno::Reference < com::sun::star::embed::XStorage > mxDocumentStorage;
};

	} // end of namespace animation
} // end of namespace sdr

//////////////////////////////////////////////////////////////////////////////

#endif //_SDR_MEDIA_MEDIAMANAGER_HXX

// eof
